require 'erb'

require 'jsonpath'
require 'json_paths'

class HTTPClient
  include Rack::Test::Methods

  def make_request(verb, path, body={})
    params = body.respond_to?(:keys) ? body : nil
    body   = body.respond_to?(:keys) ? nil : body

    env = { method: verb, format: :json }
    env[:input]  = StringIO.new(body.to_json) if body
    env[:params] = params if params

    request(path, env)
  end

  private
  def app
    Rails.application
  end
end

When(/^I make a (GET|POST|PUT|PATCH|DELETE) request to (\/.*\.json)$/) do |verb, path|
  path.gsub!(/:(\w)_id/) {|m| instance_variable_get("@#{$1}").id}
  @response = HTTPClient.new.make_request(verb, path)
end

When(/^I make a (GET|POST|PUT|PATCH|DELETE) request to (\/.*\.json) with the body:$/) do |verb, path, body|
  path.gsub!(/:(\w+)_id/) {|m| instance_variable_get("@#{$1}").id}
  body.gsub!(/:(\w+)_id/) {|m| instance_variable_get("@#{$1}").id}
  @response = HTTPClient.new.make_request(verb, path, JSON.parse(body))
end

Then(/^I should see a (\d+) status code$/) do |code|
  expect(@response.status).to eq(code.to_i)
end

Then(/^the JSON should contain:$/) do |json|
  old_json = json
  json.gsub!(/https:\/\/\w+.bleacherreport.com/, 'http://example.org')

  expected_paths   = JsonPaths.new(json)
  unexpected_paths = JsonPaths.new(@response.body).paths - expected_paths.paths

  unexpected_paths.each do |path|
    warn "The path '#{path}' was returned, but not expected"
  end

  expected_paths.paths.each do |p|
    path = JsonPath.new(p)
    expected = path.on(json)
    returned = path.on(@response.body)
    failure_message = "Got #{returned} instead of #{expected} on path #{p} in:\n#{@response.body}"

    if expected.size == 1 && expected.first.is_a?(String) && returned.first.is_a?(String)
      expected = Regexp.new(expected.first)
      returned = returned.first

      expect(returned).to match(expected), failure_message
    else
      expect(returned).to eq(expected), failure_message
    end
  end

  json = old_json
end
